from dotenv import load_dotenv
load_dotenv()

from langchain_openai import OpenAIEmbeddings
from langchain_community.vectorstores import Chroma
from langchain_community.document_loaders import TextLoader
from langchain_text_splitters import RecursiveCharacterTextSplitter
from langchain_community.document_loaders import PyPDFLoader

import os
import glob

def read_and_split_files(path='Documents', extension="pdf"):
    txt_files = glob.glob(os.path.join(path, f"*.{extension}"))
        
    doc = []
    for file_path in txt_files:
        doc.append(file_path)

    def load_and_split(file_list):
        chunks = []
        for file in file_list:
            if file.endswith(".txt"):
                loader = TextLoader(file, encoding='utf-8')
            elif file.endswith(".pdf"):
                loader = PyPDFLoader(file)
            else:
                raise ValueError(f"Unsupported file extension: {file}")

            docs = loader.load()

            # Split
            text_splitter = RecursiveCharacterTextSplitter.from_tiktoken_encoder(
                chunk_size=1000, chunk_overlap=200)
            splits = text_splitter.split_documents(docs)

            chunks.extend(splits)

        return chunks

    # Index
    docs = load_and_split(doc)
    # qa_history_doc = gen_doc_from_history()
    # docs.extend(qa_history_doc)
    # web_doc = web_data(os.path.join(path, 'web_url.csv'))
    # docs.extend(web_doc)

    return docs


def create_vectordb(docs):
    path = "../SYSTEX_精誠/RAG/Documents/"
    docs = read_and_split_files(path)

    persist_directory = 'db'

    embedding = OpenAIEmbeddings()

    vectordb = Chroma.from_documents(documents = docs,
                                    embedding = embedding,
                                    persist_directory = persist_directory)

    # 用persist方式執行vectordb，可以將db資料寫到磁碟中
    vectordb.persist()


def use_vectordb(persist_directory):
    # embedding使用OpenAI的Embedding
    embedding = OpenAIEmbeddings()

    # 使用的vectordb

    vectordb = Chroma(persist_directory=persist_directory, 
                    embedding_function=embedding)
    
def use_retriever(vectordb):
    retriever = vectordb.as_retriever(search_kwargs={'k': 2})
    ans2 = retriever.invoke('溫室氣體種類')
    print(ans2)